package pro.litvinovg.docsettings;

import java.awt.EventQueue;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.sun.star.lang.XSingleComponentFactory;

//import pro.litvinovg.docsettings.gui.ConfigurationWindow;
//import pro.litvinovg.docsettings.gui.ConversionExecutor;

import com.sun.star.lib.uno.helper.Factory;
import com.sun.star.lib.uno.helper.WeakBase;
import com.sun.star.registry.XRegistryKey;
import com.sun.star.uno.XComponentContext;

import pro.litvinovg.docsettings.document.ODFDocument;
import pro.litvinovg.docsettings.gui.MainWindow;
import pro.litvinovg.docsettings.settings.Backups;
import pro.litvinovg.docsettings.settings.Settings;

public final class DocSettingsExtension extends WeakBase
		implements com.sun.star.lang.XServiceInfo, com.sun.star.task.XJobExecutor {
	private final XComponentContext context;
	private static final String m_implementationName = DocSettingsExtension.class.getName();
	private static final String[] m_serviceNames = { "pro.litvinovg.libreoffice.DocSettings" };
	private static final Logger logger = LoggerFactory.getLogger(DocSettingsExtension.class);

	public DocSettingsExtension(XComponentContext componentContext) {
		context = componentContext;
	};

	public static XSingleComponentFactory __getComponentFactory(String sImplementationName) {
		XSingleComponentFactory xFactory = null;

		if (sImplementationName.equals(m_implementationName))
			xFactory = Factory.createComponentFactory(DocSettingsExtension.class, m_serviceNames);
		return xFactory;
	}

	public static boolean __writeRegistryServiceInfo(XRegistryKey xRegistryKey) {
		return Factory.writeRegistryServiceInfo(m_implementationName, m_serviceNames, xRegistryKey);
	}

	// com.sun.star.lang.XServiceInfo:
	public String getImplementationName() {
		return m_implementationName;
	}

	public boolean supportsService(String sService) {
		int len = m_serviceNames.length;

		for (int i = 0; i < len; i++) {
			if (sService.equals(m_serviceNames[i]))
				return true;
		}
		return false;
	}

	public String[] getSupportedServiceNames() {
		return m_serviceNames;
	}

	// com.sun.star.task.XJobExecutor:
	public void trigger(String action) {
		switch (action) {
		case "openGUI":
			try {
				MainWindow.runGUI(context);
			} catch (Throwable e) {
				DialogHelper.showErrorMessage(context, null, e.getLocalizedMessage());
			}
			break;
		case "onLoad":
			try {
				EventQueue.invokeLater(new Runnable() {
					public void run() {
						try {
							onLoad();
						} catch (Exception e) {
							DialogHelper.showErrorMessage(context, null, e.getLocalizedMessage());
						}
					}
				});
			} catch (Throwable e) {
				DialogHelper.showErrorMessage(context, null, e.getLocalizedMessage());
			}
			break;
		default:
			DialogHelper.showErrorMessage(context, null, "Unknown action: " + action);
		}
	}

	private void onLoad() {
		ODFDocument document = null;
		try {
			document = new ODFDocument(context);
		} catch (Exception e) {
			logger.debug("Ops", e);
			return;
		}
  	Backups backups = new Backups();
  	ExtensionConfig config = ExtensionConfig.getInstance(); 
		document.readSettings();
		document.readBackups(backups);
		if (!document.hasBackups(backups)) {
			backups.create(document.getCurrentSettings());
			document.writeBackups(backups);
			return;
		} 
		if (config.getAutoRestore()) {
			Settings settings = backups.getLatest();
			boolean isSettingsChanged = document.compareWithBackup(settings);
			if (isSettingsChanged) {
				document.restoreSettings(settings);
			}
		}
		
	}
}
