package pro.litvinovg.docsettings.gui;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.DefaultListModel;
import javax.swing.DefaultListSelectionModel;
import javax.swing.GroupLayout;
import javax.swing.GroupLayout.Alignment;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.LayoutStyle.ComponentPlacement;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import pro.litvinovg.docsettings.ExtensionConfig;
import pro.litvinovg.docsettings.Localizer;
import pro.litvinovg.docsettings.document.ODFDocument;
import pro.litvinovg.docsettings.settings.Backups;
import pro.litvinovg.docsettings.settings.Settings;
import javax.swing.JCheckBox;
import javax.swing.JFrame;
import javax.swing.SwingConstants;

public class BackupPanel extends JPanel {

	private ExtensionConfig config;
	private ODFDocument document;
	private Backups backups;
	private JButton btn_close;
	private JButton btn_restore;
	private JButton btn_create;
	private JButton btn_delete;
	private JList lst_backups;
	private DefaultListModel backupListModel;
	private static final Logger logger = LoggerFactory.getLogger(BackupPanel.class);
	private JCheckBox cb_stick_latest_backup;
	private JFrame frame;


	public BackupPanel(JFrame frame, ODFDocument document, Backups backups) {
		this.frame = frame;
		this.backups = backups;
		this.config = ExtensionConfig.getInstance();
		this.document = document;
		initComponents();
		initEvents();
	}
	
	private void initEvents() {
		onCreateButtonClick();
		onDeleteButtonClick();
		onRestoreButtonClick();
		onCloseButtonClick();		
		onCheckBoxClick();
	}

	private void initComponents() {
		
		backupListModel = new DefaultListModel();
		populateList();
		lst_backups = new JList(backupListModel);
		lst_backups.setSelectionMode(DefaultListSelectionModel.SINGLE_SELECTION);
		
		JLabel lb_backups = new JLabel(Localizer.get("backup_panel_description"));
		lb_backups.setHorizontalAlignment(SwingConstants.CENTER);
		
		btn_delete = new JButton(Localizer.get("delete_backup_button"));
		btn_create = new JButton(Localizer.get("create_backup_button"));
		btn_restore = new JButton(Localizer.get("restore_button"));
		btn_close = new JButton(Localizer.get("close_window_button"));
		
		cb_stick_latest_backup = new JCheckBox(Localizer.get("stick_latest_backup_cb_label"));
		cb_stick_latest_backup.setSelected(config.getAutoRestore());
		cb_stick_latest_backup.setVerticalAlignment(SwingConstants.TOP);
		
		GroupLayout groupLayout = new GroupLayout(this);
		groupLayout.setHorizontalGroup(
			groupLayout.createParallelGroup(Alignment.LEADING)
				.addGroup(groupLayout.createSequentialGroup()
					.addGap(43)
					.addGroup(groupLayout.createParallelGroup(Alignment.LEADING)
						.addComponent(lb_backups, GroupLayout.DEFAULT_SIZE, GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
						.addGroup(groupLayout.createSequentialGroup()
							.addComponent(lst_backups, GroupLayout.PREFERRED_SIZE, 247, GroupLayout.PREFERRED_SIZE)
							.addGap(53)
							.addGroup(groupLayout.createParallelGroup(Alignment.LEADING, false)
								.addComponent(cb_stick_latest_backup, GroupLayout.DEFAULT_SIZE, GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
								.addComponent(btn_close, GroupLayout.DEFAULT_SIZE, GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
								.addComponent(btn_delete, Alignment.TRAILING, GroupLayout.DEFAULT_SIZE, 269, Short.MAX_VALUE)
								.addComponent(btn_create, Alignment.TRAILING, GroupLayout.DEFAULT_SIZE, GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
								.addComponent(btn_restore, GroupLayout.DEFAULT_SIZE, GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))))
					.addGap(39))
		);
		groupLayout.setVerticalGroup(
			groupLayout.createParallelGroup(Alignment.LEADING)
				.addGroup(groupLayout.createSequentialGroup()
					.addContainerGap()
					.addComponent(lb_backups, GroupLayout.PREFERRED_SIZE, 15, GroupLayout.PREFERRED_SIZE)
					.addGap(14)
					.addGroup(groupLayout.createParallelGroup(Alignment.LEADING)
						.addComponent(lst_backups, GroupLayout.PREFERRED_SIZE, 384, GroupLayout.PREFERRED_SIZE)
						.addGroup(groupLayout.createSequentialGroup()
							.addComponent(btn_delete)
							.addGap(18)
							.addComponent(btn_create)
							.addGap(18)
							.addComponent(btn_restore)
							.addGap(33)
							.addComponent(cb_stick_latest_backup, GroupLayout.PREFERRED_SIZE, 83, GroupLayout.PREFERRED_SIZE)
							.addGap(132)
							.addComponent(btn_close)))
					.addContainerGap(74, Short.MAX_VALUE))
		);
		setLayout(groupLayout);
	}

	private void populateList() {
		backupListModel.clear();
		int i = 0;
		for(String date : backups.getBackupTimeLabels()) {
			backupListModel.add(i,date);
			i++;
		}
	}

	private void onCreateButtonClick() {
		btn_create.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				if (document == null) {
					return;
				}
				document.save();
				backups.create(document.getCurrentSettings());
				document.writeBackups(backups);
				document.save();
				populateList();
			}
		});
	}
	
	private void onDeleteButtonClick() {
		btn_delete.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				if (lst_backups.isSelectionEmpty()) {
					return;
				}
				if (document == null) {
					return;
				}
				String selected = (String) lst_backups.getSelectedValue();
				backups.removeByDateTimeLabel(selected);
				document.writeBackups(backups);
				document.save();
				populateList();
			}
		});
	}
	
	private void onRestoreButtonClick() {
		btn_restore.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				if (lst_backups.isSelectionEmpty()) {
					JOptionPane.showMessageDialog(frame, Localizer.get("no_backup_selected"));	
					return;
				}
				if (document == null) {
					JOptionPane.showMessageDialog(frame, Localizer.get("error_document_not_found"));	
					return;
				}
				String selected = (String) lst_backups.getSelectedValue();
				Settings backup;
				try {
					backup = backups.getByDateTimeLabel(selected);
					document.restoreSettings(backup);
					MainWindow.getSingleFrame().setVisible(false);
					MainWindow.getSingleFrame().dispose();
				} catch (Exception e) {
					logger.debug("Restore failed", e);
					JOptionPane.showMessageDialog(frame, Localizer.get("restore_failed \n" + e.getLocalizedMessage()));	
				}
			}
		});
	}

	private void onCloseButtonClick() {
		btn_close.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				MainWindow.getSingleFrame().setVisible(false);
				MainWindow.getSingleFrame().dispose();
			}
		});
	}
	
	private void onCheckBoxClick() {
		cb_stick_latest_backup.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				config.setAutoRestore(cb_stick_latest_backup.isSelected());
			}
		});
	}
}
