package pro.litvinovg.docsettings.gui;

import java.awt.EventQueue;
import java.awt.Toolkit;

import javax.swing.JFrame;
import javax.swing.JOptionPane;

import java.awt.GridLayout;
import javax.swing.JTabbedPane;
import javax.swing.UIManager;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.sun.star.uno.XComponentContext;

import pro.litvinovg.docsettings.DialogHelper;
import pro.litvinovg.docsettings.DocSettingsExtension;
import pro.litvinovg.docsettings.ExtensionConfig;
import pro.litvinovg.docsettings.Localizer;
import pro.litvinovg.docsettings.document.ODFDocument;
import pro.litvinovg.docsettings.settings.Backups;

import javax.swing.JPanel;
import java.awt.BorderLayout;
import java.awt.FlowLayout;
import javax.swing.BoxLayout;
import javax.swing.GroupLayout;
import javax.swing.GroupLayout.Alignment;

import javax.swing.JList;

public class MainWindow extends JFrame  {

	private static final long serialVersionUID = 1L;
	private static JFrame frame = null;
	private Localizer localizer;
	private ODFDocument document;
	private Backups backups;
	private static final Logger logger = LoggerFactory.getLogger(MainWindow.class);


	public static void main(String[] args) {
		EventQueue.invokeLater(new Runnable() {
			public void run() {
				try {
					MainWindow window = new MainWindow();
					frame = window;
				} catch (Exception e) {
					e.printStackTrace();
				}
			}
		});
	}

	public static void runGUI(XComponentContext context) {

		if (frame != null) {
			frame.dispose();
		}
			try {
				UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
			} catch (Throwable e) {
				String message = e.getLocalizedMessage();
				JOptionPane.showMessageDialog(frame, message);	
			} 
			EventQueue.invokeLater(new Runnable() {
				public void run() {
					try {
						MainWindow window = new MainWindow(context);
						frame = window;
					} catch (Throwable e) {
						DialogHelper.showErrorMessage(context, null, e.getLocalizedMessage());
						logger.debug("Ops", e);
					} 
				}
			});	
		
	}
	
	public static JFrame getSingleFrame() {
		return frame;
	}
	
	public MainWindow() {
		backups = new Backups();
		initialize();
	}

	public MainWindow(XComponentContext context) throws Exception {
		document = new ODFDocument(context);
		backups = new Backups();
		document.readSettings();
		document.readBackups(backups);
		initialize();
	}

	/**
	 * Initialize the contents of the frame.
	 */
	private void initialize() {
		setIconImage(Toolkit.getDefaultToolkit().getImage(MainWindow.class.getResource("/pro/litvinovg/docsettings/resources/icon.png")));
		setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
		getContentPane().setLayout(new GridLayout(1, 0, 0, 0));
		localizer = Localizer.getInstance();
		this.setBounds(100, 100, 750, 500);
		this.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
		this.setVisible(true);

		JTabbedPane tabbedPane = new JTabbedPane(JTabbedPane.TOP);
		getContentPane().add(tabbedPane);
		
		JPanel backupPanel = new BackupPanel(this, document, backups);
		tabbedPane.addTab(localizer.getTranslation("backups_tab_name"), null, backupPanel, null);
		
		//JPanel settingsPanel = new SettingsPanel(this, document);
		//tabbedPane.addTab(localizer.getTranslation("settings_tab_name"), null, settingsPanel, null);
		


	}
}
