package pro.litvinovg.docsettings.settings;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class ComparatorResults {
	
	private static final Logger logger = LoggerFactory.getLogger(ComparatorResults.class);

	private List<ODFSetting> removedSettings = new ArrayList<ODFSetting>();
	private List<ODFSetting> newSetting = new ArrayList<ODFSetting>();
	private Map<ODFSetting,ODFSetting> changedSettings = new HashMap<ODFSetting,ODFSetting>();
	
	public boolean hasChanges() {
		return removedSettings.isEmpty() && newSetting.isEmpty() && changedSettings.isEmpty();
	}
	
	public void addRemovedProperty(ODFSetting setting){
		removedSettings.add(setting);
	}
	
	public void addNewProperty(ODFSetting setting){
		newSetting.add(setting);
	}
	
	public void addChangedProperty(ODFSetting oldSetting,ODFSetting newSetting){
		changedSettings.put(oldSetting, newSetting);
	}

	public List<ODFSetting> getRemovedProperties() {
		return removedSettings;
	}

	public List<ODFSetting> getAddedProperties() {
		return newSetting;
	}

	public Map<ODFSetting, ODFSetting> getChangedProperties() {
		return changedSettings;
	}

	public void log() {
		logger.info("New settings:");
		for (Iterator iterator = newSetting.iterator(); iterator.hasNext();) {
			ODFSetting odfSetting = (ODFSetting) iterator.next();
			logger.debug(odfSetting.getName());
		}
		logger.info("Removed settings:");
		for (Iterator iterator = removedSettings.iterator(); iterator.hasNext();) {
			ODFSetting odfSetting = (ODFSetting) iterator.next();
			logger.info(odfSetting.getName());
		}
		logger.info("Changed settings:");
		for (Entry<ODFSetting, ODFSetting> entry : changedSettings.entrySet()) {
			logger.info(entry.getKey().getName() + "\nChanged from "+ entry.getKey().getValue() + " to " + entry.getValue().getValue() );
		}
	}
	
}
