package pro.litvinovg.docsettings.settings;

public class SettingsComparator {

	private static final String RSID = "Rsid";

	public static boolean isEqual(Settings currentSettings, Settings backedUpSettings) {
		Settings newSettings = new Settings();
		newSettings.addAll(currentSettings);
		Settings oldSettings = new Settings();
		oldSettings.addAll(backedUpSettings);
		ComparatorResults results = compare(newSettings, oldSettings);
		if (results.hasChanges()) {
			results.log();
		}
		return !results.hasChanges();
	}

	private static ComparatorResults compare(Settings newSettings, Settings oldSettings) {
		ComparatorResults results = new ComparatorResults();
		while (!newSettings.isEmpty()) {
			ODFSetting newSet = newSettings.get(0);
			ODFSetting oldSet = oldSettings.getByName(newSet.getName());
			if (oldSet == null) {
				results.addNewProperty(newSet);
			} else {
				if (!(newSet.getValue()).equals(oldSet.getValue()) && !isExcluded(newSet.getName())) {
					results.addChangedProperty(oldSet, newSet);	
				} 
				oldSettings.remove(oldSet);
			}
			newSettings.remove(newSet);
		}
		while (!oldSettings.isEmpty()) {
			ODFSetting oldSet = oldSettings.get(0);
			results.addRemovedProperty(oldSet);
		}
		return results;
	}

	private static boolean isExcluded(String name) {
		if (name.equals(RSID)) {
			return true;
		}
		return false;
	}

}
