package pro.litvinovg.docsettings.settings;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;

import pro.litvinovg.docsettings.exceptions.MalformedXmlException;

public class SimpleSetting implements ODFSetting {

	private static final Logger logger = LoggerFactory.getLogger(SimpleSetting.class);
	protected String name;
	protected String value;
	protected String type;

	public SimpleSetting(Node node) throws MalformedXmlException {
		this.name = getNodeName(node);
		this.value = node.getTextContent();
		this.type = node.getAttributes().getNamedItem("config:type").getNodeValue();
	}
	protected String getNodeName(Node node) throws MalformedXmlException {
		 Node nameAttr = node.getAttributes().getNamedItem("config:name");
		 if (nameAttr == null) {
			 logger.debug("name attribute not found while creating simple property");
			 throw new MalformedXmlException("Node " + node.getNodeName() + " config:name attribute not found.");
		 }
		return nameAttr.getNodeValue();
	}
	
	@Override
	public String getName() {
		return name;
	}
	
	@Override
	public String getValue() {
		return value;
	}
	@Override
	public String getType() {
		return type;
	}
	@Override
	public void addToDocumentElement(Document doc, Node rootElement) {
		Element element = doc.createElement("config:config-item");
		element.setAttribute("config:name", getName());
		element.setAttribute("config:type", getType());
		element.setTextContent(getValue());
		rootElement.appendChild(element);
	}
}
