package pro.litvinovg.docsettings.settings;

import java.io.IOException;
import java.io.StringReader;
import java.io.StringWriter;
import java.util.Base64;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathExpression;
import javax.xml.xpath.XPathExpressionException;
import javax.xml.xpath.XPathFactory;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

import pro.litvinovg.docsettings.exceptions.MalformedXmlException;

public class StorageUtils {
	
	private static final Logger logger = LoggerFactory.getLogger(StorageUtils.class);
	
	public static Document getDocFromString(String xmlString) throws ParserConfigurationException, SAXException, IOException {
		DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
		DocumentBuilder db = dbf.newDocumentBuilder();
		Document doc = db.parse(new InputSource(new StringReader(xmlString)));
		return doc;
	}
	
	public static Document createNewDoc() throws ParserConfigurationException {
		DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
		DocumentBuilder db = dbf.newDocumentBuilder();
		Document doc = db.newDocument();
		return doc;
	}
	
	public static String serializeDoc(Document doc) throws TransformerException {
    StringWriter output = new StringWriter();
		Transformer transformer = TransformerFactory.newInstance().newTransformer();
		transformer.setOutputProperty(OutputKeys.ENCODING, "UTF-8"); 
    transformer.transform(new DOMSource(doc), new StreamResult(output));
    String str = output.toString();
		return str;
	}
	
	public static String printNode(Node node) throws TransformerException {
    StringWriter output = new StringWriter();
		Transformer transformer = TransformerFactory.newInstance().newTransformer();
		transformer.setOutputProperty(OutputKeys.ENCODING, "UTF-8"); 
    transformer.transform(new DOMSource(node), new StreamResult(output));
    String str = output.toString();
		return str;
	}
	
	
	public static String encodeBase64(String string) {
		return new String(Base64.getEncoder().encode(string.getBytes()));
	}
	
	public static String decodeBase64(String serializedMetaSettings) {
		return new String(Base64.getDecoder().decode(serializedMetaSettings));
	}
	
	public static void removeNodes(NodeList nodes) {
		for (int i = nodes.getLength()-1; i >= 0; i--) {
			Node node = nodes.item(i);
			node.getParentNode().removeChild(node);
		}
	}
	
	public static void appendSettingNodesToDoc(Settings settings, Document doc, Node root) {
		for(int i = 0; i< settings.getSize();i++) {
			ODFSetting setting = settings.get(i);
			setting.addToDocumentElement(doc,root);
		}
	}
	
	public static Node getSettingsElement(Document doc) throws Exception {
		XPathFactory xPathfactory = XPathFactory.newInstance();
		XPath xpath = xPathfactory.newXPath();
		XPathExpression expr = xpath.compile("//*[local-name() ='config-item-set'][@*[local-name() ='name' and .='ooo:configuration-settings']]");
		NodeList configItems = (NodeList) expr.evaluate(doc, XPathConstants.NODESET);
		Node configSettings = configItems.item(0);
		if (configSettings == null) {
			throw new MalformedXmlException("No configuration settings element found");
		}
		return configSettings;
	}
	
	public static Node getMetaRootElement(Document doc) throws MalformedXmlException {
		XPathFactory xPathfactory = XPathFactory.newInstance();
		XPath xpath = xPathfactory.newXPath();
		XPathExpression expr;
		NodeList nodes;
		try {
			expr = xpath.compile("//*[local-name() ='meta']");
			nodes = (NodeList) expr.evaluate(doc, XPathConstants.NODESET);
			if (nodes.getLength() == 0 ) {
				throw new MalformedXmlException("Xpath expression failed while looking for main meta element");
			}
		} catch (XPathExpressionException e) {
			logger.debug(e.getLocalizedMessage());
			logger.debug("Ops!",e);
			throw new MalformedXmlException("Xpath expression failed while looking for main meta element");
		}
		Node result = nodes.item(0);
		return result;
	}
	
	

}
